Package: brglm2
Title: Bias Reduction in Generalized Linear Models
Version: 1.1.0
Authors@R: c(person(given = "Ioannis", family = "Kosmidis", role = c("aut", "cre"), email = "ioannis.kosmidis@warwick.ac.uk", comment = c(ORCID = "0000-0003-1556-0302")),
	     person(given = c("Euloge", "Clovis"), family = c("Kenne Pagui"), role = "aut", email = "kenne@stat.unipd.it", comment = c(ORCID = "0000-0002-8998-9251")),
 	     person(given = "Federico", family = "Boiocchi", role = "ctb", email = "federico.boiocchi@gmail.com"),
	     person(given = "Philipp", family = "Sterzinger", role = "ctb", email = "P.Sterzinger@lse.ac.uk", comment = c(ORCID = "0009-0007-7348-5810")),
             person(given = "Kjell", family = "Konis", role = "ctb", email = "kjell.konis@me.com"),
	     person(given = "Nicola", family = "Sartori", role = "ctb", email = "sartori@stat.unipd.it"))
Description: Estimation and inference from generalized linear models based on various methods for bias reduction and maximum penalized likelihood with powers of the Jeffreys prior as penalty. The 'brglmFit()' fitting method can achieve reduction of estimation bias by solving either the mean bias-reducing adjusted score equations in Firth (1993) <doi:10.1093/biomet/80.1.27> and Kosmidis and Firth (2009) <doi:10.1093/biomet/asp055>, or the median bias-reducing adjusted score equations in Kenne et al. (2017) <doi:10.1093/biomet/asx046>, or through the direct subtraction of an estimate of the bias of the maximum likelihood estimator from the maximum likelihood estimates as in Cordeiro and McCullagh (1991) <https://www.jstor.org/stable/2345592>. See Kosmidis et al (2020) <doi:10.1007/s11222-019-09860-6> for more details. Estimation in all cases takes place via a quasi Fisher scoring algorithm, and S3 methods for the construction of of confidence intervals for the reduced-bias estimates are provided. In the special case of generalized linear models for binomial and multinomial responses (both ordinal and nominal), the adjusted score approaches to mean and media bias reduction have been found to return estimates with improved frequentist properties, that are also always finite, even in cases where the maximum likelihood estimates are infinite (e.g. complete and quasi-complete separation; see Kosmidis and Firth, 2020 <doi:10.1093/biomet/asaa052>, for a proof for mean bias reduction in logistic regression). The 'mdyplFit()' fitting method fits logistic regression models using maximum Diaconis-Ylvisaker prior penalized likelihood, which also guarantees finite estimates. High-dimensionality corrections under proportional asymptotics can be applied to the resulting objects; see Sterzinger and Kosmidis (2024) <doi:10.48550/arXiv.2311.07419> for details.
URL: https://github.com/ikosmidis/brglm2
BugReports: https://github.com/ikosmidis/brglm2/issues
Depends: R (>= 3.5)
Imports: MASS, stats, Matrix, graphics, nnet, enrichwith, numDeriv,
        statmod, nleqslv
License: GPL-3
Encoding: UTF-8
LazyData: true
RoxygenNote: 7.3.3
Suggests: detectseparation, knitr, rmarkdown, covr, tinytest, VGAM,
        brglm
VignetteBuilder: knitr
NeedsCompilation: yes
Packaged: 2026-04-29 13:30:08 UTC; yiannis
Author: Ioannis Kosmidis [aut, cre] (ORCID:
    <https://orcid.org/0000-0003-1556-0302>),
  Euloge Clovis Kenne Pagui [aut] (ORCID:
    <https://orcid.org/0000-0002-8998-9251>),
  Federico Boiocchi [ctb],
  Philipp Sterzinger [ctb] (ORCID:
    <https://orcid.org/0009-0007-7348-5810>),
  Kjell Konis [ctb],
  Nicola Sartori [ctb]
Maintainer: Ioannis Kosmidis <ioannis.kosmidis@warwick.ac.uk>
Repository: CRAN
Date/Publication: 2026-04-29 14:40:02 UTC
