# Generated by roxygen2: do not edit by hand

S3method(add1,brglmFit)
S3method(add1,mdyplFit)
S3method(coef,bracl)
S3method(coef,brglmFit)
S3method(coef,brglmFit_expo)
S3method(coef,brmultinom)
S3method(coef,brnb)
S3method(confint,brglmFit)
S3method(confint,brmultinom)
S3method(confint,brnb)
S3method(confint,mdyplFit)
S3method(drop1,brglmFit)
S3method(drop1,mdyplFit)
S3method(expo,brglmFit)
S3method(expo,glm)
S3method(fitted,bracl)
S3method(fitted,brmultinom)
S3method(logLik,brmultinom)
S3method(model.matrix,brmultinom)
S3method(ordinal_superiority,bracl)
S3method(plrtest,mdyplFit)
S3method(predict,bracl)
S3method(predict,brmultinom)
S3method(print,anova_mdyplFit)
S3method(print,brglmFit_expo)
S3method(print,brmultinom)
S3method(print,brnb)
S3method(print,summary.bracl)
S3method(print,summary.brglmFit)
S3method(print,summary.brmultinom)
S3method(print,summary.brnb)
S3method(print,summary.mdyplFit)
S3method(residuals,bracl)
S3method(residuals,brmultinom)
S3method(simulate,brmultinom)
S3method(simulate,brnb)
S3method(summary,bracl)
S3method(summary,brglmFit)
S3method(summary,brmultinom)
S3method(summary,brnb)
S3method(summary,mdyplFit)
S3method(vcov,bracl)
S3method(vcov,brglmFit)
S3method(vcov,brmultinom)
S3method(vcov,brnb)
export(bracl)
export(brglmControl)
export(brglmFit)
export(brglm_control)
export(brglm_fit)
export(brmultinom)
export(brnb)
export(check_infinite_estimates)
export(detect_separation)
export(expo)
export(mdyplControl)
export(mdyplFit)
export(mdypl_control)
export(mdypl_fit)
export(mis)
export(ordinal_superiority)
export(plrtest)
export(se0)
export(se1)
export(sloe)
export(solve_se)
import(MASS)
import(enrichwith)
import(nleqslv)
import(stats)
import(Matrix)
importFrom(graphics,plot)
importFrom(nnet,class.ind)
importFrom(numDeriv,grad)
importFrom(statmod,gauss.quad)
useDynLib(brglm2)
